using System;
using System.IO;
using NUnit.Framework;
using ThoughtWorks.CruiseControl.Core.Util;

namespace ThoughtWorks.CruiseControl.Core.Sourcecontrol.Test
{
	[TestFixture]
	public class VaultHistoryParserTest : CustomAssertion
	{
		private const string XML = 
			@"<vault>
				<history>
					<item txid=""2"" date=""5/13/2003 10:41:30 PM"" name=""$"" type=""70"" version=""1"" user=""admin"" comment=""creating repository"" />
				</history>
			  </vault>";
		private const string NO_COMMENT_XML =
			@"<vault>
				<history>
					<item txid=""2"" date=""5/13/2003 10:41:30 PM"" name=""$"" type=""70"" version=""1"" user=""admin"" />
				</history>
			  </vault>";

		private StringReader GetReader(string xml)
		{
			return new StringReader(xml);
		}

		[Test]
		public void TestNumberOfModifications()
		{
			VaultHistoryParser parser = new VaultHistoryParser();
			StringReader reader = GetReader(XML);
			Modification[] modifications = parser.Parse(reader, new DateTime(2003, 5, 12), DateTime.Now);
			reader.Close();
			AssertEquals(1, modifications.Length);
		}

		[Test]
		public void TestNumberOfModificationsWithInvalidDate()
		{
			VaultHistoryParser parser = new VaultHistoryParser();
			StringReader reader = GetReader(XML);
			Modification[] modifications = parser.Parse(reader, DateTime.Now.AddMinutes(-1), DateTime.Now);
			reader.Close();
			AssertEquals(0, modifications.Length);
		}

		[Test]
		public void TestModificationData()
		{
			VaultHistoryParser parser = new VaultHistoryParser();
			StringReader reader = GetReader(XML);
			Modification[] modifications = parser.Parse(reader, new DateTime(2003, 5, 12), DateTime.Now);
			reader.Close();
			Modification mod = modifications[0];
			AssertEquals("$", mod.FolderName);
			AssertEquals(null, mod.FileName);
			AssertEquals(new DateTime(2003, 5, 13, 22, 41, 30), mod.ModifiedTime);
			AssertEquals("Created", mod.Type);
			AssertEquals("admin", mod.UserName);
			AssertEquals(2, mod.ChangeNumber);
		}

		/// <summary>
		/// Tests a history entry with no comments (bug in first release of these classes).
		/// </summary>
		/// <remarks>
		/// Apparently the "comments" attribute is not always in the XML.
		/// </remarks>
		[Test]
		public void NoCommentsTest()
		{
			VaultHistoryParser parser = new VaultHistoryParser();
			StringReader reader = GetReader(NO_COMMENT_XML);
			Modification[] modifications = parser.Parse(reader, new DateTime(2003, 5, 12), DateTime.Now);
			reader.Close();
		}
	}
}
